//*************************************************
// Utilizzo di una matrice a led 4x4
// con soli 3 I/O tramite 74HC595
// www.settorezero.com
//
// modulo: main.c
// autore: Bernardo Giovanni
// copyright: 2010 Bernardo Giovanni
// data: 12/11/2010
// picmicro: PIC16F877A
// clock: 20MHz
//
//*************************************************

#define  XTAL_FREQ 20MHZ
#include <pic.h>

__CONFIG (HS & WDTDIS & PWRTEN & BORDIS & LVPDIS & DUNPROT & WRTEN & DEBUGDIS & UNPROTECT);

#include "delay.c" // routine per ritardi
#include "settings.h" // settaggi del picmicro

void main(void)
	{
	settings(); // eseguo settaggi
	
	while(1) //ciclo finito
        {

		// primo ciclo: accendo i led da 0 a 7
		// accendo prima il led 0, poi i led 0 e 1, poi i led 0,1 e 2, poi i led 0,1,2 e 3 ecc
		for (a=0; a<8; a++)
			{
			scan[0]=scan[0]^(1<<a);
			DelayMs(250);
			}
		
		// secondo ciclo come sopra
		// ma per i led da 8 a 15
		for (a=0; a<8; a++)
			{
			scan[1]=scan[1]^(1<<a);
			DelayMs(250);
			}

		// finiti i due cicli spengo tutti i led
		scan[0]=0;
		scan[1]=0;
		DelayMs(250);
		}// Fine ciclo continuo
	
	} // Fine main

	
// interrupt service routine
void interrupt ISR (void)
	{
	if (T0IF) // L'interrupt  stato causato da un overflow del timer0 ?
		{
		TMR0 = 100; // Reimposto Timer0
		
		// ogni 5mS cambio la colonna:
		// 0001
		// 0010
		// 0100
		// 1000
		
		// controlliamo la colonna da visualizzare
		switch(multiplex_column)	
			{
			case 0:
				// mi prendo il nibble basso di scan[0]
				// ovvero i valori di riga da accendere per la colonna 1
				tempscan=scan[0] & 0b00001111;
			break;
			
			case 1:
				// mi prendo il nibble alto di scan[0]
				// ovvero i valori di riga da accendere per la colonna 2
				tempscan=scan[0] >> 4;
			break;
			
			case 2:
				// mi prendo il nibble basso di scan[1]
				// ovvero i valori di riga da accendere per la colonna 3
				tempscan=scan[1] & 0b00001111;
			break;
			
			case 3:
				// mi prendo il nibble alto di scan[1]
				// ovvero i valori di riga da accendere per la colonna 4
				tempscan=scan[1] >> 4;
			break;
			}
		
		// sposto l'1 nella posizione della colonna desiderata
		// in maniera da attivare la colonna
		latchOut=0b00010000 << multiplex_column;
		
		// sommo il valore della colonna da visualizzare
		// il nibble alto contiene il valore di colonna da accendere
		// il nibble basso contiene i numeri di riga da visualizzare
		// per questa colonna
		latchOut |= tempscan;
		// ora latchOut contiene il giusto valore da inviare al SIPO
		
		multiplex_count++; // incremento il conteggio dei mS per la visualizzazione
		
		if (multiplex_count==MPC) // raggiunti gli MPC millisecondi
			{
			// invio i dati allo shift register
			// il primo dato inviato va a finire nel bit 7 e no nello zero!!!
			// per cui dobbiamo scansionare latchOut dal MSB all'LSB
			for (shift=0; shift<8; shift++)
				{	
				value = latchOut & (0b10000000 >> shift);
				if (value)
					{
					DATA=1;
					}
				else
					{
					DATA=0;
					}
				// la transizione del clock dello shift register da alto a basso
				// trasferisce il dato in ingresso nello shift register
				CLOCK=1;
				CLOCK=0;
				}
			// la transizione del clock del latch da alto a basso
			// trasferisce tutto il byte dallo shift register al latch
			LATCH=1;
			LATCH=0;
			
			 // azzero il counter di visualizzazione e...
			multiplex_count=0;
			// ... e passo alla colonna successiva
			multiplex_column++;
			
			if (multiplex_column==4) // abbiamo raggiunto l'ultima colonna quindi...
				{
				multiplex_column=0; // ...ripartiamo dalla prima
				}
			}
		
		T0IF=0; // Resetto il flag interrupt su timer 0,
		} // fine che interrupt verificatosi su timer0
	} // fine interrupt service routine