//*************************************************
// Utilizzo di una matrice a led 4x4
// con soli 3 I/O tramite 74HC595
// www.settorezero.com
//
// modulo: settings.h
// autore: Bernardo Giovanni
// copyright: 2010 Bernardo Giovanni
// data: 12/11/2010
// picmicro: PIC16F877A
// clock: 20MHz
//
//*************************************************

#define DATA	RC1 // linea dati su RC1
#define CLOCK	RC3 // linea clock shift register su RC3
#define LATCH	RC5 // linea clock latch su RC5
#define MPC		4 // 4 mS di multiplex

unsigned char scan[]={0,0}; // scan[0] e scan[1] contengono lo stato dei led
unsigned char latchOut=0; // dato in uscita per 74HC595
unsigned char shift=0; // contatore usato per serializzazione
unsigned char value=0; // variabile di appoggio usata per serializzazione
unsigned char a=0; // counter generico
unsigned char multiplex_count=0; // conteggia i millisecondi di multiplex
unsigned char multiplex_column=0; // colonna attualmente scansionata
unsigned char tempscan=0; // valore temporaneo

void settings(void)
	{
	// Tutte le porte come output 
	TRISA=0;
	TRISB=0;
	TRISC=0;
	TRISD=0;
	TRISE=0;
	
	// Impostazione del registro OPTION (pag.55 del datasheet)
	OPTION=0b11000100;
	// bit 0 -> Prescaler Rate Select bit 0
	// bit 1 -> Prescaler Rate Select bit 0
	// bit 2 -> Prescaler Rate Select bit 0 (1:32)
	// bit 3 -> Prescaler assegnato al Timer0 
	// bit 4 -> Non importa
	// bit 5 -> Clock per Timer0 derivato da ciclo di clock interno
	// bit 6 -> Non importa
	// bit 7 -> Resistenze di pull-up su porta B disattivate
	
	// Impostazione Interrupt
	INTCON=0b10100000;
	// bit 0 -> RBIF - Flag interrupt su porte B
	// bit 1 -> INTF - Flag interrupt su RB0/INT
	// bit 2 -> T0IF - Flag interrupt su Timer0
	// bit 3 -> RBIE, Interrupt su porte B disattivato
	// bit 4 -> INTE, Interrupt su porta RB0/INT disattivato
	// bit 5 -> TMR0IE, Interrupt su Timer0 attivato
	// bit 6 -> PEIE, Interrupt di periferica disattivato
	// bit 7 -> GIE, Gestione Interrupt attiva
	
	TMR0=100; // Imposto Timer0 a 100
	}